@@ -1,5 +1,6 @@
 import builtins
 from datetime import datetime, timezone
+import logging
 import os.path
 from pathlib import Path
 import random
@@ -12,6 +13,7 @@ from dandischema.models import UUID_PATTERN, DigestType, get_schema_version
 import pytest
 import responses
 
+from .skip import mark
 from .. import dandiapi
 from ..consts import (
     DRAFT,
@@ -59,6 +61,7 @@ def test_publish_and_manipulate(local_dandi_api, monkeypatch, tmp_path):
         },
     )
     dandiset_id = d.identifier
+    assert str(d) == f"DANDI-API-LOCAL-DOCKER-TESTS:{dandiset_id}/draft"
     upload_dir = tmp_path / "upload"
     upload_dir.mkdir()
     (upload_dir / dandiset_metadata_file).write_text(f"identifier: '{dandiset_id}'\n")
@@ -75,7 +78,11 @@ def test_publish_and_manipulate(local_dandi_api, monkeypatch, tmp_path):
     )
 
     d.wait_until_valid()
-    version_id = d.publish().version.identifier
+    v = d.publish().version
+    version_id = v.identifier
+    assert str(v) == version_id
+    dv = d.for_version(v)
+    assert str(dv) == f"DANDI-API-LOCAL-DOCKER-TESTS:{dandiset_id}/{version_id}"
 
     download_dir = tmp_path / "download"
     download_dir.mkdir()
@@ -86,10 +93,7 @@ def test_publish_and_manipulate(local_dandi_api, monkeypatch, tmp_path):
     dandiset_yaml = download_dir / dandiset_id / dandiset_metadata_file
     file_in_version = download_dir / dandiset_id / "subdir" / "file.txt"
 
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions/{version_id}",
-        download_dir,
-    )
+    download(dv.version_api_url, download_dir)
     assert downloaded_files() == [dandiset_yaml, file_in_version]
     assert file_in_version.read_text() == "This is test text.\n"
 
@@ -102,10 +106,7 @@ def test_publish_and_manipulate(local_dandi_api, monkeypatch, tmp_path):
         validation="skip",
     )
     rmtree(download_dir / dandiset_id)
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions/{version_id}",
-        download_dir,
-    )
+    download(dv.version_api_url, download_dir)
     assert downloaded_files() == [dandiset_yaml, file_in_version]
     assert file_in_version.read_text() == "This is test text.\n"
 
@@ -119,10 +120,7 @@ def test_publish_and_manipulate(local_dandi_api, monkeypatch, tmp_path):
     )
 
     rmtree(download_dir / dandiset_id)
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions/draft",
-        download_dir,
-    )
+    download(d.version_api_url, download_dir)
     assert sorted(downloaded_files()) == [
         dandiset_yaml,
         file_in_version,
@@ -132,28 +130,19 @@ def test_publish_and_manipulate(local_dandi_api, monkeypatch, tmp_path):
     assert file_in_version.with_name("file2.txt").read_text() == "This is more text.\n"
 
     rmtree(download_dir / dandiset_id)
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions/{version_id}",
-        download_dir,
-    )
+    download(dv.version_api_url, download_dir)
     assert downloaded_files() == [dandiset_yaml, file_in_version]
     assert file_in_version.read_text() == "This is test text.\n"
 
     d.get_asset_by_path("subdir/file.txt").delete()
 
     rmtree(download_dir / dandiset_id)
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions/draft",
-        download_dir,
-    )
+    download(d.version_api_url, download_dir)
     assert downloaded_files() == [dandiset_yaml, file_in_version.with_name("file2.txt")]
     assert file_in_version.with_name("file2.txt").read_text() == "This is more text.\n"
 
     rmtree(download_dir / dandiset_id)
-    download(
-        f"{local_dandi_api['instance'].api}/dandisets/{dandiset_id}/versions/{version_id}",
-        download_dir,
-    )
+    download(dv.version_api_url, download_dir)
     assert downloaded_files() == [dandiset_yaml, file_in_version]
     assert file_in_version.read_text() == "This is test text.\n"
 
@@ -163,6 +152,7 @@ def test_get_asset_metadata(local_dandi_api, simple1_nwb):
     d = client.create_dandiset(name="Include Metadata Test", metadata={})
     d.upload_raw_asset(simple1_nwb, {"path": "testing/simple1.nwb", "foo": "bar"})
     (asset,) = d.get_assets()
+    assert str(asset) == f"DANDI-API-LOCAL-DOCKER-TESTS:assets/{asset.identifier}"
     metadata = asset.get_raw_metadata()
     assert metadata["path"] == "testing/simple1.nwb"
     assert metadata["foo"] == "bar"
@@ -270,6 +260,7 @@ def test_authenticate_bad_key_keyring_good_key_input(
     confirm_mock.assert_called_once_with("API key is invalid; enter another?")
 
 
+@mark.skipif_no_network
 def test_get_content_url(tmp_path):
     with DandiAPIClient.for_dandi_instance("dandi") as client:
         asset = client.get_dandiset("000027", "draft").get_asset_by_path(
@@ -288,6 +279,7 @@ def test_get_content_url(tmp_path):
                 fp.write(chunk)
 
 
+@mark.skipif_no_network
 def test_get_content_url_regex(tmp_path):
     with DandiAPIClient.for_dandi_instance("dandi") as client:
         asset = client.get_dandiset("000027", "draft").get_asset_by_path(
@@ -300,6 +292,7 @@ def test_get_content_url_regex(tmp_path):
                 fp.write(chunk)
 
 
+@mark.skipif_no_network
 def test_get_content_url_follow_one_redirects_strip_query():
     with DandiAPIClient.for_dandi_instance("dandi") as client:
         asset = client.get_dandiset("000027", "draft").get_asset_by_path(
@@ -317,6 +310,7 @@ def test_remote_asset_json_dict(text_dandiset):
     assert asset.json_dict() == {
         "asset_id": anys.ANY_STR,
         "modified": anys.ANY_AWARE_DATETIME_STR,
+        "created": anys.ANY_AWARE_DATETIME_STR,
         "path": anys.ANY_STR,
         "size": anys.ANY_INT,
     }
@@ -579,3 +573,75 @@ def test_refresh(text_dandiset):
     assert dandiset.version_id == DRAFT
     assert dandiset.version.modified > mtime
     assert dandiset.most_recent_published_version is not None
+
+
+def test_get_asset_with_and_without_metadata(mocker, text_dandiset):
+    path_asset = text_dandiset["dandiset"].get_asset_by_path("file.txt")
+    id_asset = text_dandiset["dandiset"].get_asset(path_asset.identifier)
+    assert path_asset == id_asset
+    assert path_asset._metadata is None
+    assert id_asset._metadata is not None
+    get_spy = mocker.spy(text_dandiset["client"], "get")
+    id_metadata = id_asset.get_raw_metadata()
+    get_spy.assert_not_called()
+    path_metadata = path_asset.get_raw_metadata()
+    get_spy.assert_called_once()
+    assert path_metadata == id_metadata
+
+
+@responses.activate
+def test_retry_logging(caplog):
+    responses.add(responses.GET, "https://test.nil/api/info/", status=503)
+    responses.add(responses.GET, "https://test.nil/api/info/", status=503)
+    responses.add(responses.GET, "https://test.nil/api/info/", json={"foo": "bar"})
+    client = DandiAPIClient("https://test.nil/api")
+    assert client.get("/info/") == {"foo": "bar"}
+    responses.assert_call_count("https://test.nil/api/info/", 3)
+    assert (
+        "dandi",
+        logging.DEBUG,
+        "GET https://test.nil/api/info/",
+    ) in caplog.record_tuples
+    assert (
+        "dandi",
+        logging.WARNING,
+        "Retrying GET https://test.nil/api/info/",
+    ) in caplog.record_tuples
+    assert (
+        "dandi",
+        logging.INFO,
+        "GET https://test.nil/api/info/ succeeded after 2 retries",
+    ) in caplog.record_tuples
+    assert ("dandi", logging.DEBUG, "Response: 200") in caplog.record_tuples
+
+
+def test_get_assets_order(text_dandiset):
+    assert [
+        asset.path for asset in text_dandiset["dandiset"].get_assets(order="path")
+    ] == ["file.txt", "subdir1/apple.txt", "subdir2/banana.txt", "subdir2/coconut.txt"]
+    assert [
+        asset.path for asset in text_dandiset["dandiset"].get_assets(order="-path")
+    ] == ["subdir2/coconut.txt", "subdir2/banana.txt", "subdir1/apple.txt", "file.txt"]
+
+
+def test_get_assets_with_path_prefix(text_dandiset):
+    assert sorted(
+        asset.path
+        for asset in text_dandiset["dandiset"].get_assets_with_path_prefix("subdir")
+    ) == ["subdir1/apple.txt", "subdir2/banana.txt", "subdir2/coconut.txt"]
+    assert sorted(
+        asset.path
+        for asset in text_dandiset["dandiset"].get_assets_with_path_prefix("subdir2")
+    ) == ["subdir2/banana.txt", "subdir2/coconut.txt"]
+    assert [
+        asset.path
+        for asset in text_dandiset["dandiset"].get_assets_with_path_prefix(
+            "subdir", order="path"
+        )
+    ] == ["subdir1/apple.txt", "subdir2/banana.txt", "subdir2/coconut.txt"]
+    assert [
+        asset.path
+        for asset in text_dandiset["dandiset"].get_assets_with_path_prefix(
+            "subdir", order="-path"
+        )
+    ] == ["subdir2/coconut.txt", "subdir2/banana.txt", "subdir1/apple.txt"]
